%% Demo: Robust PCA for background-foreground separation of videos
% Task:
%   We apply Alternating Direction Method of Multipliers (ADMM) to solve this model,
% where the augmented Lagrangian is given by:
%   L(A,B,M,Y) = |A|_nuc + lambda*|B|_l1 + delta{|M-Z|_fro <= eps} +
%   <Y, A+B-M> + 0.5*rho*|A+B-M|_fro^2
%   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   


%% problem setup
clear all
close all hidden

rng(1)

addpath('./util');
    
% number of frames (max 200), use small number for testing.
nFrame=20;

data_formation

%% show the original video
% until here, you should get a matrix Z with n1*n2
Z_show = imnorm(reshape(Z,[z1 z2 z3]));
for i = 1:nFrame
    imshow(Z_show(:,:,i));
    pause(1/35);
end

%% ADMM parameters
lambda = .5/sqrt(n1);     % parameter of l1 norm on B
eps = .01;                % threshold for ||M-Z||_fro
rho = 1;                  % parameter in ADMM
max_it = 100;             % max number of iterations
energy = zeros(1,max_it); % array used to record energy of augmented Lagrangian

%% TODO: initialization of A, B, M, Y


%% TODO: implement ADMM solver
for iter = 1:max_it
    % update A:
    
    % update B:
    
    % update M:
    
    % update Y:

    % compute and record energy:

end


%% show result
A = imnorm(reshape(A,[z1 z2 z3]));
B = imnorm(reshape(B,[z1 z2 z3]));
Z = imnorm(reshape(Z,[z1 z2 z3]));
figure(1)
for i = 1:nFrame
   subplot(1,3,1)
   subimage(A(:,:,i));
   title('background')
   subplot(1,3,2)
   subimage(B(:,:,i));
   title('foreground')
   subplot(1,3,3)
   subimage(Z(:,:,i));
   title('original');
   pause(1/35)
end
